-- NoAlienChat allows you to see cross-faction chat as well as other messages normally hidden behind "alien" font.
-- It doesn't affect profanity filter.
-- version 3.3

require "Apollo"
require "GameLib"

local addon = 
{
	opt = {},
	opt_default =
	{
		scramble_enemy = false,
		scramble_alien = false,
		enemy_prefix = "",
		tag_alien = true,
		enemy_color = "red"
	},
	supported_addons = {"ChatLog", "ChatFixed", "ImprovedChatLog", "BetterChatLog", "FixedChatLog", "Killroy", "ChatAdvanced", "ShortChannelName"},
	present_addons = {}
}

for k, v in pairs(addon.opt_default) do
	addon.opt[k] = v
end

for k, v in pairs(addon.supported_addons) do
	addon.present_addons[v] = true
end
addon.present_addons["ShortChannelName"] = nil	-- no need to hook that one

function addon:OnDependencyError(dep, err)
	self.present_addons[dep] = nil
	return true
end

function addon:OnSave(lvl)
	if lvl ~= GameLib.CodeEnumAddonSaveLevel.Character then
		return
	end
	
	local opt = {}
	
	-- ignoring defaults, so we can overwrite them in a later version
	for k, v in pairs(self.opt) do
		if v ~= self.opt_default[k] then
			opt[k] = v
		end
	end
	
	return opt
end

function addon:OnRestore(lvl, opt)
	if lvl ~= GameLib.CodeEnumAddonSaveLevel.Character then
		return
	end
	
	for k, v in pairs(opt) do
		if self.opt[k] ~= nil then
			self.opt[k] = v
		end
	end
end

function addon:post_msg(msg)
	ChatSystemLib.PostOnChannel(ChatSystemLib.ChatChannel_Command, msg, "NoAlienChat")
end

function addon:on_slash_cmd(cmd, s)
	local k, v = s:match("([^=]+)=(.*)")
	
	if k and self.opt[k] ~= nil then
		if type(self.opt[k]) == "boolean" then
			local bool_lut = {
				["0"] = false,
				["1"] = true,
				["false"] = false,
				["true"] = true
			}
			
			if bool_lut[v] ~= nil then
				self.opt[k] = bool_lut[v]
			else
				self:post_msg("Bad value (" .. v .. "). Try this:\n/" .. cmd .. " " .. k .. "=" .. tostring(not self.opt[k]))
			end
		elseif type(self.opt[k]) == "string" then
			self.opt[k] = v
		end
	else
		local opt_str = " \nUsage: /" .. cmd .. " option=value\nCurrent options:\n"
		for k, v in pairs(self.opt) do
			local default
			local default_v = self.opt_default[k]
			
			if v == default_v then
				default = " (default)"
			else
				default = " (default: " .. (type(default_v) == "string" and "\"" .. default_v .. "\"" or tostring(default_v)) .. ")"
			end
			
			opt_str = opt_str .. "(" .. type(v) .. ") " .. tostring(k) .. "=" .. (type(v) == "string" and "\"" .. v .. "\"" or tostring(v)) .. default .. "\n"
		end
		self:post_msg(opt_str)
	end
end

function addon:process_msg(tMessage)
	local show_alien = not self.opt.scramble_alien
	local tag_alien = self.opt.tag_alien
	
	if tMessage.bCrossFaction and not self.opt.scramble_enemy then
		tMessage.bCrossFaction = false
		
		local enemy_prefix = self.opt.enemy_prefix
		if enemy_prefix ~= "" then
			table.insert(tMessage.arMessageSegments, 1, {strText = enemy_prefix})
		end
	end
	
	if show_alien then
		for idx, tSegment in ipairs( tMessage.arMessageSegments ) do
			if tSegment.bAlien then
				tSegment.bAlien = false
				if tag_alien then
					tSegment.strText = "{#}" .. tSegment.strText .. "{#}"
				end
			end
		end
	end
	
	return tMessage
end

function addon:change_color(tQueuedMessage)
	local t = tQueuedMessage.xml:ToTable()
	for i = 1, #t[1] do
		if t[1][i].__XmlNode == "Source" then
			t[1][i].TextColor = self.opt.enemy_color
			break
		end
	end
	tQueuedMessage.xml = XmlDoc.CreateFromTable(t)
end

function addon:replace_helper(target)
	local old_f = target.HelperGenerateChatMessage
	local self2 = self
	
	function target:HelperGenerateChatMessage(tQueuedMessage)
		if tQueuedMessage.xml then
			return
		end
		
		local bCrossFaction = tQueuedMessage.tMessage.bCrossFaction
		
		self2:process_msg(tQueuedMessage.tMessage)
		old_f(self, tQueuedMessage)
		
		if tQueuedMessage.xml and bCrossFaction and self2.opt.enemy_color ~= "" then
			self2:change_color(tQueuedMessage)
		end
	end
end

function addon:OnLoad()
	if not next(self.present_addons) then
		error("No supported addons were found.", 0)
	end
	
	Apollo.RegisterSlashCommand("noalienchat", "on_slash_cmd", self)
	
	for k in pairs(self.present_addons) do
		local target = Apollo.GetAddon(k)
		if k == "ChatAdvanced" then	-- Addon author declares no dependencies and calls OnRestore himself. God help him.
			local old_f = target.OnRestore
			local self2 = self
			
			function target:OnRestore(...)
				old_f(self, ...)
				if self.ChatLog then
					self2:replace_helper(self.ChatLog)
				end
			end
		elseif k == "Killroy" then
			local old_f = target.Change_HelperGenerateChatMessage
			local self2 = self
			
			function target:Change_HelperGenerateChatMessage()
				old_f(self)
				self2:replace_helper(Apollo.GetAddon("ChatLog"))
			end
		else
			self:replace_helper(target)
		end
	end
end

Apollo.RegisterAddon(addon, false, "", addon.supported_addons)
